<?php

namespace Drupal\Tests\builder_notes\Functional;

use Drupal\Core\Entity\EntityDisplayBase;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\node\Traits\ContentTypeCreationTrait;

/**
 * Tests builder expected_notes functionality.
 *
 * @group builder_notes
 */
class BuilderNotesTest extends BrowserTestBase {

  use ContentTypeCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'filter',
    'options',
    'builder_notes',
    'field',
    'field_ui',
  ];

  public function testBuilderNotesUI() {
    $this->drupalLogin($this->drupalCreateUser([
      'administer content types',
      'administer nodes',
      'administer node form display',
      'administer node display',
      'access administration pages',
      'view the administration theme',
    ]));
    $this->createContentType(['type' => 'page']);
    $this->drupalGet('admin/structure/types/manage/page/display');
    $notes = 'I am going to use the default display in the search results';
    $this->assertNotesField($notes);
    $this->drupalGet('admin/structure/types/manage/page/form-display');
    $this->assertNotesField($notes);
    /** @var \Drupal\Core\Entity\Entity\EntityFormDisplay $form_display */
    $form_display = $this->container->get('entity_type.manager')->getStorage('entity_form_display')->load('node.page.default');
    $this->assertNotesSaved($notes, $form_display);
    /** @var \Drupal\Core\Entity\Entity\EntityViewDisplay $view_display */
    $view_display = $this->container->get('entity_type.manager')->getStorage('entity_view_display')->load('node.page.default');
    $this->assertNotesSaved($notes, $view_display);
  }

  /**
   * Assert that expected_notes field works.
   *
   * @param string $notes
   *   Notes to use.
   */
  protected function assertNotesField($notes) {
    $assert = $this->assertSession();
    $field_name = 'builder_notes';
    $assert->fieldExists($field_name);
    $this->submitForm([
      $field_name => $notes,
    ], 'Save');
    $assert->fieldValueEquals($field_name, $notes);
  }

  /**
   * Assert display expected_notes were saved.
   *
   * @param string $expected_notes
   *   Expected note.
   * @param \Drupal\Core\Entity\EntityDisplayBase $display
   *   Display on which expected_notes should be present.
   */
  protected function assertNotesSaved($expected_notes, EntityDisplayBase $display) {
    $this->assertEquals($expected_notes, $display->getThirdPartySetting('builder_notes', 'notes'));
  }

}
